-- UUAT_UpgradeTriggerer
-- Author: HandyVac
-- DateCreated: 2/17/2016 11:42:57 AM
--------------------------------------------------------------
local Harmony_ID = GameInfo.Affinity_Types["AFFINITY_TYPE_HARMONY"].ID;
local Purity_ID = GameInfo.Affinity_Types["AFFINITY_TYPE_PURITY"].ID;
local Supremacy_ID = GameInfo.Affinity_Types["AFFINITY_TYPE_SUPREMACY"].ID;



function UUAT_ListenForUpgradeTechs (iTeam, iTech, bAdopted)
	-- function is called per team, so process all players in this team
	for UnwantedThing, pPlayer in pairs(Players) do
		if pPlayer:IsAlive() and pPlayer:GetTeam() == iTeam then
			print("UUAT: Checking for Upgrade Unlocks. (" .. pPlayer:GetName() .. ")"); 
			
			-- get name of the tech that was just researched
			local strTechType = GameInfo.Technologies[iTech].Type;
			print("Researched: " .. strTechType);

			-- loop through UUAT table and apply any upgrades triggered by this tech
			for trigger in GameInfo.UnitUpgrade_UUAT_TechTriggers() do
				if strTechType == trigger.TechType then

					-- lookup relevant info about the upgrade
					local iUnit_ID = GameInfo.Units[trigger.UnitType].ID;
					local iPerk_ID = GameInfo.UnitPerks[trigger.PerkType].ID;
					local iUpgrade_ID = -1;

					print("... this triggers an upgrade for " .. trigger.UnitType .. "(ID: " .. iUnit_ID .. ") including perk " .. trigger.PerkType .. "(ID: " .. iPerk_ID .. ")");

									
					-- get the number of upgrades this unit has recieved already (if any)
					local iNumUpgradesSoFar = table.count( pPlayer:GetUpgradesForUnit(iUnit_ID) );
					print(trigger.UnitType .. " has " .. iNumUpgradesSoFar .. " upgrades already");
					
					-- get list of all posible upgrades it could recieve for next tier up
					local UpgradesTable = {};
					local iUpgradeCounter = 0;

					for upgrade in GameInfo.UnitUpgrades() do
						if upgrade.UnitType == trigger.UnitType and upgrade.UpgradeTier == iNumUpgradesSoFar +1 then
							print("... " .. upgrade.Type);
							UpgradesTable[iUpgradeCounter] = upgrade;
							iUpgradeCounter = iUpgradeCounter +1;
						end
					end

					print("There are " .. iUpgradeCounter .. " possible upgrades " .. trigger.UnitType .. " could gain at tier " .. iNumUpgradesSoFar + 1);

					if iUpgradeCounter == 0 then
						-- if there are no possible upgrades, then we cannot upgrade this unit
						print("... There are no possible upgrades to choose");

						-- TO DO: Send a notification to the player telling them upgrades are maxed out.

					elseif iUpgradeCounter == 1 then
						-- if there's only one upgrade, use that one
						print("... Chosen: " .. UpgradesTable[0].Type .. " (ID: " .. UpgradesTable[0].ID .. ")");
						iUpgrade_ID = UpgradesTable[0].ID;
						-- note: iUpgradeCounter will always be one higher then the last table index
					else
						-- if there's more than one potential upgrade, choose which one to grant based on player's current affinity

						-- find the 1 letter codes for player's current dominant and runner-up affinities
						local strPrimeAffin, strSecondAffin = UUAT_GetHybridAffinityCode(pPlayer);

						if strPrimeAffin == "" or strPrimeAffin == nil then
							print("UUAT Error: Player has no dominant affinity");
						else
							print("Player's dominant affinity is " .. strPrimeAffin .. ", secondary is " .. strSecondAffin);
							--look through the list of possibilities for one that matches the player's hybrid affinity
							 for iUpgrade = 0, (iUpgradeCounter -1) do
								local strTestedCode = string.match(UpgradesTable[iUpgrade].Type, "UNITUPGRADE_.-_[0-9](.+)");
								if strTestedCode == (strPrimeAffin .. strSecondAffin) or strTestedCode == (strSecondAffin.. strPrimeAffin) then
									iUpgrade_ID = UpgradesTable[iUpgrade].ID;
									print("... Chosen: " .. UpgradesTable[iUpgrade].Type .. " (ID: " .. UpgradesTable[iUpgrade].ID .. ")");
								end
							end
							-- if we can't fnd a hybrid upgrade, look for one that matches their dominant affinity only
							if iUpgrade_ID == -1 then
								print("... no upgrades for " .. strPrimeAffin .. strSecondAffin .. " or ".. strSecondAffin .. strPrimeAffin .. ", looking for " .. strPrimeAffin);
								for iUpgrade = 0, (iUpgradeCounter -1) do
									if string.match(UpgradesTable[iUpgrade].Type, "UNITUPGRADE_.-_[0-9](.+)") == strPrimeAffin then
										iUpgrade_ID = UpgradesTable[iUpgrade].ID;
										print("... Chosen: " .. UpgradesTable[iUpgrade].Type .. " (ID: " .. UpgradesTable[iUpgrade].ID .. ")");
									end
								end
							end
							-- if we still can't find one that matches their dominant affinity, print an error message
							if iUpgrade_ID == -1 then
								print("UUAT Error: None of the available upgrades is valid for the player's affinity")
							end
						end
					end

					-- Apply the upgrade
					if iUpgrade_ID ~= -1 then
						print("applying upgrade (Unit ID: " .. iUnit_ID .. ", Upgrade ID: " .. iUpgrade_ID .. ", Perk ID: " .. iPerk_ID .. ")");
						pPlayer:AssignUnitUpgrade( iUnit_ID, iUpgrade_ID, iPerk_ID );

			
						-- Send a notification to the player, if human
						print("Sending notification to the player");

						local TestString = "Test";
						local Str_NotifHeader = Locale.ConvertTextKey("TXT_KEY_UUAT_NOTIF_UNIT_UPGRADED_HEADER", GameInfo.Units[iUnit_ID].Description );
						local Str_NotifBody = Locale.ConvertTextKey("TXT_KEY_UUAT_NOTIF_UNIT_UPGRADED_BODY", GameInfo.Units[iUnit_ID].Description, GameInfo.UnitUpgrades[iUpgrade_ID].Description, trigger.PerkHelpTechweb );				
						
						pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, Str_NotifBody, Str_NotifHeader );
						print("Sent notification to the player")
					end
				end
			end
		end
	end
end

-- call this whenever any player aquires a tech
GameEvents.TeamSetHasTech.Add(UUAT_ListenForUpgradeTechs);
-- ################################################
-- ##             Utility Functions              ##
-- ################################################

-- returns the code string for the input player's dominant affinity/hybrid affinity.
function UUAT_GetHybridAffinityCode (pPlayer)
-- fraction of dominant affinity needed to qualify as a secondary (hybrid) affinity
	local iHybridThreshold = 0.75;

	-- string to return
	local strPrimaryAffin = "";
	local strSecondaryAffin = "";

	-- get this player's dominant affinity
	local iDominantAffinity_ID = pPlayer:GetDominantAffinityType();

	-- get scores for each of the three primary affinities
	local iHarmonyScore = pPlayer:GetAffinityLevel(Harmony_ID);
	local iPurityScore = pPlayer:GetAffinityLevel(Purity_ID);
	local iSupremacyScore = pPlayer:GetAffinityLevel(Supremacy_ID);

	if iDominantAffinity_ID == Harmony_ID then
		strPrimaryAffin = "H";
		-- check for Harmony-Purity hybrid
		if iPurityScore > iSupremacyScore and iPurityScore > (iHarmonyScore * iHybridThreshold) then
			strSecondaryAffin = "P"
		end
		-- check for Harmony-Supremacy hybrid
		if iSupremacyScore > iPurityScore and iSupremacyScore > (iHarmonyScore * iHybridThreshold) then
			strSecondaryAffin = "P"
		end
	elseif iDominantAffinity_ID == Purity_ID then
		strPrimaryAffin = "P";
		-- check for Purity-Harmony hybrid
		if iHarmonyScore > iSupremacyScore and iHarmonyScore > (iPurityScore * iHybridThreshold) then
			strSecondaryAffin = "H"
		end
		-- check for Purity-Supremacy hybrid
		if iSupremacyScore > iHarmonyScore and iSupremacyScore > (iPurityScore * iHybridThreshold) then
			strSecondaryAffin = "S"
		end
	elseif iDominantAffinity_ID == Supremacy_ID then
		strPrimaryAffin = "S";
		-- check for Supremacy-Harmony hybrid
		if iHarmonyScore > iSupremacyScore and iHarmonyScore > (iSupremacyScore * iHybridThreshold) then
			strSecondaryAffin = "H"
		end
		-- check for Supremacy-Purity hybrid
		if iPurityScore > iSupremacyScore and iPurityScore > (iSupremacyScore * iHybridThreshold) then
			strSecondaryAffin = "P"
		end
	else
		print("UUAT: Player has no dominant affinity");
	end
	
	-- return both strings
	return strPrimaryAffin, strSecondaryAffin;
end