-- This script will automatically generate dummy PlayerPerks and place them on the techweb
-- as labels to mark which techs trigger UnitUpgrades

-- ### Step (1) Generate the perks ###

INSERT INTO PlayerPerks (Type, Help, IconAtlas, IconIndex)
	SELECT	("PLAYERPERK_UUAT_LABEL_" || ID), 
			("TXT_KEY_PLAYERPERK_UUAT_LABEL_" || ID), 
			( SELECT PortraitAtlas FROM ArtDefine_UnitInfos WHERE Type LIKE ( (SELECT UnitArtInfo FROM Units WHERE Type = UnitType) || '%') ORDER BY Type LIMIT 1 ),
			( SELECT PortraitIndex FROM ArtDefine_UnitInfos WHERE Type LIKE ( (SELECT UnitArtInfo FROM Units WHERE Type = UnitType) || '%') ORDER BY Type LIMIT 1 )
	FROM UnitUpgrade_UUAT_TechTriggers;

-- note: the LIKE and wildcard are neccesary because some units have more than one art define (depending on afinity/level)


-- ### Step (2) place the perks on the techweb ###

INSERT INTO Technology_FreePlayerPerks (TechType, PlayerPerkType)
	SELECT TechType, ("PLAYERPERK_UUAT_LABEL_" || ID)
	FROM UnitUpgrade_UUAT_TechTriggers;


-- ### Step (3) Generate text key descriptions for the perks ###

-- for Upgrade Triggers with a PerkHelpTechweb entry in UnitUpgrade_UUAT_TechTriggers
INSERT INTO Language_en_US (Tag, Text) 
	SELECT	("TXT_KEY_PLAYERPERK_UUAT_LABEL_" || ID), 
			(
			(SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_BEGINNING') || 
			(SELECT Text FROM Language_en_US WHERE Tag = (SELECT Description FROM Units WHERE Type = UnitType))
			|| (SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_MIDDLE') || 
			(SELECT Text FROM Language_en_US WHERE Tag = PerkHelpTechweb )
			|| (SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_END')
			)
	FROM UnitUpgrade_UUAT_TechTriggers WHERE PerkHelpTechweb IS NOT NULL AND (SELECT Text FROM Language_en_US WHERE Tag = PerkHelpTechweb) IS NOT NULL;

-- Alternative tooltip if TXT_KEY enrty is missing. Useful for bug hunting.
INSERT INTO Language_en_US (Tag, Text) 
	SELECT	("TXT_KEY_PLAYERPERK_UUAT_LABEL_" || ID), 
			(
			(SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_BEGINNING') || 
			(SELECT Text FROM Language_en_US WHERE Tag = (SELECT Description FROM Units WHERE Type = UnitType))
			|| (SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_MIDDLE') || 
			PerkHelpTechweb
			|| (SELECT Text FROM Language_en_US WHERE Tag='TXT_KEY_UUAT_TRIGGER_LABEL_END')
			)
	FROM UnitUpgrade_UUAT_TechTriggers WHERE PerkHelpTechweb IS NOT NULL AND (SELECT Text FROM Language_en_US WHERE Tag = PerkHelpTechweb) IS NULL;



-- ### Step (4) Add Military flavours to unit-upgrading techs, so the AI knows it should research them. ###

-- Offense, defence and Military Training flavours for all units
INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_OFFENSE'), (10 + (SELECT Combat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers;

INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_DEFENSE'), (10 + (SELECT Combat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers;

INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_MILITARY_TRAINING'), (10)
	FROM UnitUpgrade_UUAT_TechTriggers;

-- Ranged flavour for ranged units
INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_RANGED'), (10 + (SELECT RangedCombat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers WHERE (SELECT CombatClass FROM Units WHERE Type = UnitType) IN ('UNITCOMBAT_ARCHER', 'UNITCOMBAT_SIEGE', 'UNITCOMBAT_NAVALRANGED', 'UNITCOMBAT_SUBMARINE');

-- Mobile flavour for fast moving units
INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_MOBILE'), (10 + (SELECT Combat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers WHERE (SELECT CombatClass FROM Units WHERE Type = UnitType) IN ('UNITCOMBAT_RECON','UNITCOMBAT_MOUNTED', 'UNITCOMBAT_ARMOR', 'UNITCOMBAT_HELICOPTER');

-- Naval flavour for sea units
INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_NAVAL'), (10 + (SELECT Combat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers WHERE (SELECT CombatClass FROM Units WHERE Type = UnitType) IN ('UNITCOMBAT_NAVALRANGED', 'UNITCOMBAT_NAVALMELEE', 'UNITCOMBAT_SUBMARINE', 'UNITCOMBAT_CARRIER');

-- Air flavour for air units
INSERT INTO Technology_Flavors (TechType, FlavorType, Flavor)
	SELECT (TechType), ('FLAVOR_AIR'), (10 + (SELECT Combat FROM Units WHERE Type = UnitType))
	FROM UnitUpgrade_UUAT_TechTriggers WHERE (SELECT CombatClass FROM Units WHERE Type = UnitType) IN ('UNITCOMBAT_HELICOPTER', 'UNITCOMBAT_FIGHTER', 'UNITCOMBAT_BOMBER', 'UNITCOMBAT_CARRIER');



-- TO DO: Add database triggers to redo all of this if another mod changes UnitUpgrade_UUAT_TechTriggers table.


-- TO DO: add string of "upgrades these units: ... " to help tooltips of techs that trigger upgrades.